/*-----------------------------------------------------------------------------
FILE    : crsutil.h
CONTENT : headers for auxilliary functions used in command line toolbox crs   
AUTHOR  : Frank Rügheimer
HISTORY : 2012-02-18 branched from psvmodel to provide modular interface
                     added E_STDINCNT error message and code
          2012-04-23 added E_OPTARGDSC_2 message
-------------------------------------------------------------------------------
 Copyright 2012 Frank Rügheimer
 This program is distributed under the terms of the GNU Lesser General Public
 License.
-----------------------------------------------------------------------------*/

#ifndef CRS_UTIL
#define CRS_UTIL

/* configuration variables */
#define SC_SCAN     /* required for enabling  parser definitions */
#define AS_PARSE    /* enable attribute set parsing              */

#include <assert.h>
#include <stdio.h> 
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <stdarg.h>
#include <math.h>
#include <errno.h>
#include "tabscan.h"         /* Christian Borgelt's lib for parsing tables */
#include "attset.h"          /* .. attribute management                    */
#include "io.h"              /* .. domain/header file support              */
#include "parse.h"           /* .. and parsing/scanning text files         */
#include "hdist.h"           /* Core library for hierarch. cond. rand sets */ 
#include "frdebug.h"         /*       debugging and test support functions */

/* return codes for readnextsv() */
#define  ESV_NOERROR 0
#define  ESV_ELCNT -1
#define  ESV_BUFOVRN -2
#define  ESV_UNKNOWN -3
#define  ESV_NODATA -4

/*---shared error codes for main programs---*/
/* for new messages add parameter count */
#define E_NONE  0         /* no error*/
#define E_OPENF 1         /* failed to open file */
#define E_EOF 2           /* unexpected EOF*/
#define E_ATTVAL 3        /* unknown attribute value*/
#define E_READFILE 4
#define E_OPTION 5        /* unknown option*/
#define E_ARGCNT 6        /* invalid number of arguments */
#define E_NO_MEM 7        /* out of memeory */
#define E_PARSETREE 8     /* failure to parse tree: */
#define E_PARSETBL 9      /* error parsing table file: */
#define E_OPTARG 10       /* missing option argument */
#define E_BMODE 11        /* invalid branching mode option argument: %s */
#define E_OPTARG_M 12     /* invalid argument for -m option: %s */
#define E_LOGZERO 13      /* zero prob. while computing log-likelihood ...
...actually that is okay -- -inf is a valid result - no longer used */
#define E_LCORR 14        /* invalid value for laplace correction */
#define E_PARSE 15        /* error while parsing file: %s */
#define E_STDINCNT 16     /* only one file can be assigned to stdin() */
#define E_OPTARGDSC_2 17  /* invalid argument for %s option: %s */
#define E_READFRAME 18    /* error while reading frame specification */
#define E_UNKNOWN 19      /* unknown error */

/* improving readibility */
#define FALSE 0
#define TRUE 1


/*-----------------------------------------------------------------------------
  general purpose functions for argument & error handling                  
-----------------------------------------------------------------------------*/

typedef struct {      /* for comfortable access to option params */
  int code;           /* numeric code */
  char* name;         /* name */
} MINFO;              /* from C.B. */

/*-----------------------------------------------------------------------------
  conventions for progams globals
-----------------------------------------------------------------------------*/
char *progname;        /* to be filled with program name           */
void cleanup();        /* a cleanup function to be called on error */

extern void error(int code, ... );                    /* error interpreter */
extern int code (const MINFO *tab, const char *name); /* string lookup     */
extern int readnext_sv(char* buffer, int bufsize, char** tokens, int maxel,\
 TABSCAN* scanner, FILE* input);                    /* instantiation parser */
#endif
