/*-----------------------------------------------------------------------------
File    : frameconv.h
Contents: header file for frameconv demo/tool
          definitions for reference frames (frames of discernment)
          definition of function headers (not exported)
Author  : Frank Rgheimer
History : 20.02.2007 file created as pdist.h
        : 23.02.2007 probability tree components 
        : 25.02.2007 added hdist interface 
        : 25.02.2007 switched to using attset.h for attribute management
        : 26.02.2007 added reference frame and valnametoids
        : 26.02.2007 fixed makefile
        : 27.02.2007 added probabilistic conversion between refernce frames
        : 19.08.2007 read tree structure from external files
	: 21.08.2007 added reading forest structure from external files
        : 22.08.2007 added dot-like graph language support :
        : 28.08.2007 switched to node based tree representation 
        : 31.08.2007 adapted parsing and tree construction to new nodes
        : 04.09.2007 support for set-valued distributions added
        : 08.09.2007 added branch modes for treating non-leaf nodes
        : 09.09.2007 moved hdist code to seperate files 
          2010-05-02 renamed to frameconv.h
                     minor mods to code and comments
          2010-10-01 adapted to new table package

-----------------------------------------------------------------------------*/

#define SC_SCAN     /* required for enabling  parser definitions */

#include <assert.h>
#include <stdio.h> 
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <stdarg.h>
#include "tabscan.h" /* Christian Borgelt's lib for parsing tables */
#include "attmap.h" /* .. attribute management                    */
#include "parse.h"  /* .. and parsing/scanning text files         */ 



/*---structs---*/

typedef struct _refframe{ /*represenation of a reference frame  */
  int size;    /* number of attribute's visible  values in the reference frame*/
  int *sel;    /* index map of attribute ids of a hdist */
} REFFRAME;

/*---function declarations---*/


#if false
REFFRAME* rf_create(const HDIST *hdist,const int* sel, int size);
void rf_show(REFFRAME *rf, HDIST *hdist);
void rf_delete(REFFRAME* rf);

int valnamestoids(ATT* att,const char **names, int *ids, int size);
int convdist(const HDIST *hdist, const REFFRAME *src,\
const double *srcdist, const REFFRAME *dst, double* destdist); 
int main(int argc, char* argv[]);

#endif
