/*-----------------------------------------------------------------------------
File    : psvmodel.h
Contents: various methods of modelling set-valued distributions
Author  : Frank Rgheimer
History : 15.12.2008 file created (adapted from ptree)
          17.12.2008 replaced re-definitions by ptree import
          21.12.2008 implemented file parsing for model construction
                     bug in parsing causes segfault with some artf. data
          29.12.2008 compiled on different machine - bug disappared (compiler
                     related?)
          29.12.2008 added readnext_sv() to separate parsing from model constr.
          30.12.2008 modified readnext_sv() to facilitate counting records
          04.01.2008 enabled attribute set parsing support
          04.01.2008 changed signature of imodel_induce
          2010-10-01 adapted to new table package version
          2012-05-17 some modifications in conjunction to pushing for const 
                     correctness in hdist 2.x.x
-----------------------------------------------------------------------------*/

#define SC_SCAN     /* required for enabling  parser definitions */
#define AS_PARSE    /* enable attribute set parsing */

#include <assert.h>
#include <stdio.h> 
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <stdarg.h>
#include <math.h>
#include "tabscan.h"         /* Christian Borgelt's lib for parsing tables */
#include "attset.h"          /* .. attribute management                    */
#include "io.h"              /* .. domain/header file support              */
#include "parse.h"           /* .. and parsing/scanning text files         */ 
/*  include "hdist.h"  */    /* reuse model structures I defined in ptree  */



/* return codes for readnextsv() */
#define  ESV_NOERROR 0
#define  ESV_ELCNT -1
#define  ESV_BUFOVRN -2
#define  ESV_UNKNOWN -3
#define  ESV_NODATA -4

/* data stuctures for simple models (for comparison) */

typedef struct {
  ATT *att;        /* for managing values */
  int vsz;         /* size of internal data vectors */ 
  /* use att->valcount to access the true number of values */
  double* value;   /* coverage frequency for values */ 
  int reccnt;      /* count total number of record read (for laplace corr.)*/
} IMODEL;


/* shared auxilliary functions */
/* scan file and record values of the last  set-valued attribute in the next record */
/* return value is either TFS_REC, TFS_EOF,  or a (negative) error code. */
int readnext_sv(char* buffer,int bufsize, char** tokens, int maxel,\
 TABSCAN* scanner, FILE* input);
IMODEL* imodel_induce(TABSCAN* scanner, FILE* trndata,ATT* eatt);
void imodel_delete(IMODEL* imodel);
int imodel_test(IMODEL* imodel, TABSCAN* scanner, FILE* tstdata);
