/*-----------------------------------------------------------------------------
FILE    : test_io.c
CONTENTS: test program for v2.0 extension to hdist
AUTHOR  : Frank Rügheimer
STATUS  : intentionally uses hard-coded paths - and static error messaged
          this progam is not targeted at end-users and not designed for re-use
HISTORY : 31-08-2010 file created
          01-10-2010 adapted to new table package version
-----------------------------------------------------------------------------*/
/*---includes---*/
#include <errno.h>
/* include "psvmodel.h" */
#include "hdist.h"


/*--error hndlg */


/*---main program---*/
int  main(int argc, char* argv[]){
  /* char *fn_dom="ex/test.dom"; */
  /*
  char *fn_htr="ex/example.htr";
  char *fn_tab="ex/examsv.tab";
  /*/
  char *fn_htr="ex/test4.htr";
  char *fn_tab="ex/test4.tab"; 
  
  char *fn_sav="ex/test_io.sav";
  FILE* f;
  HDIST *hd   = NULL;               /* hierarchical distribution model */
  /* ATT   *att  = NULL; */         /* base attribute */
  SCAN  *scan = NULL;
  TABSCAN *scanner=NULL;            /* table file scanner*/
  scan=sc_create(fn_htr); 
  if (!scan) {
    fprintf(stderr,"could not read file %s\n",fn_htr); exit(1); 
    /* error(E_NOMEM); */
    /* error(E_OPENF,fn_dom); */
  }
  pa_init(scan);                /* use parser ext. for reading hdist */
  hd=hdist_parse(scan,HDBM_NXHTV);
  /* reading data */
  f=fopen(fn_tab,"r");
  if (!f) {
    fprintf(stderr,"could not open file %s\n",fn_tab);
    exit(1);
  }
  scanner=ts_create(); if (!scanner) fprintf(stderr,"out of memory\n");
  /* configure file scanner for csc and tsv format */
  ts_chars(scanner,TS_FLDSEP,",\t");/* set field separators, ...        */
  ts_chars(scanner,TS_RECSEP,"\n"); /* ...and record separator          */ 
  hdist_getfreqs(hd,scanner,f);
  if (f) fclose(f); f=NULL;
  if (scanner) ts_delete(scanner); scanner=NULL;
  hdist_print(hd);
  fprintf(stderr,"---\nsaving model to file %s...\n----\n",fn_sav);
  f=fopen(fn_sav,"w");
  if (!f) {
    fprintf(stderr,"could not write to file %s\n",fn_sav);
    exit(1);
  }
  hdist_save(hd,f);
  if (f) {fclose(f); f=NULL;}
  hdist_delete(hd);
  fprintf(stderr,"clearing distribution model...\n");
  fprintf(stderr,"loading distribution model from %s\n",fn_sav);
  if (scan) sc_delete(scan);         /* reinitialize file parser */
  scan=sc_create(fn_sav);
  if (!scan) {
     fprintf(stderr,"out of memory or failed to open save file\n");
     exit(1);
  }
  pa_init(scan);
  hd=hdist_load(scan);
  if (hd){
    fprintf(stdout,"mode is %i; lcorr is %f\n",hd->bmode,hd->lcorr);
    hdist_print(hd);
    hdist_delete(hd);
  }
  if (scan) sc_delete(scan);
  return 0;
}


